﻿using BlacksmithWorkshopContracts.BindingModels;
using BlacksmithWorkshopContracts.BusinessLogicContracts;
using BlacksmithWorkshopContracts.SearchModels;
using BlacksmithWorkshopContracts.StoragesContracts;
using BlacksmithWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace BlacksmithWorkShopBusinessLogic.BusinessLogics
{
    public class ManufactureLogic : IManufactureLogic
    {
        private readonly ILogger _logger;
        private readonly IManufactureStorage _manufactureStorage;
        public ManufactureLogic(ILogger<ManufactureLogic> logger, IManufactureStorage manufactureStorage)
        {
            _logger = logger;
            _manufactureStorage = manufactureStorage;
        }
        public List<ManufactureViewModel>? ReadList(ManufactureSearchModel? model)
        {
            _logger.LogInformation("Readlist. ManufactureName: {ManufactureName}. Id: {Id}", model?.ManufactureName, model?.Id);
            var list = model == null ? _manufactureStorage.GetFullList() : _manufactureStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("Readlist. Count: {Count}", list.Count);
            return list;
        }
        public ManufactureViewModel? ReadElement(ManufactureSearchModel? model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. ManufactureName: {ManufactureName}. Id: {Id}", model.ManufactureName, model.Id);
            var element = _manufactureStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement find. Id: {Id}", element.Id);
            return element;
        }
        public bool Create(ManufactureBindingModel model)
        {
            CheckModel(model);
            if (_manufactureStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }
        public bool Update(ManufactureBindingModel model)
        {
            CheckModel(model);
            if (_manufactureStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        public bool Delete(ManufactureBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id:{Id}", model.Id);
            if (_manufactureStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(ManufactureBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            if (string.IsNullOrEmpty(model.ManufactureName))
            {
                throw new ArgumentNullException("Нет названия изделия", nameof(model.ManufactureName));
            }
            if (model.Price <= 0)
            {
                throw new ArgumentNullException("Цена изделия должна быть больше 0", nameof(model.Price));
            }
            _logger.LogInformation("Manufacture. ManufactureName: {ManufactureName}. Price: {Price}. Id: {Id}", model.ManufactureName, model.Price, model.Id);
            var element = _manufactureStorage.GetElement(new ManufactureSearchModel
            {
                ManufactureName = model.ManufactureName
            });
            if (element != null && element.Id != model.Id)
            {
                throw new InvalidOperationException("Изделие с таким названием уже есть");
            }
        }
    }
}